"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.callStepFile = void 0;
const step_handler_rpc_processor_1 = require("./step-handler-rpc-processor");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const nodeRunner = path_1.default.join(__dirname, 'node', 'node-runner.js');
const pythonRunner = path_1.default.join(__dirname, 'python', 'python-runner.py');
const rubyRunner = path_1.default.join(__dirname, 'ruby', 'ruby_runner.rb');
const getLanguageBasedRunner = (stepFilePath = '') => {
    const isPython = stepFilePath.endsWith('.py');
    const isRuby = stepFilePath.endsWith('.rb');
    const isNode = stepFilePath.endsWith('.js') || stepFilePath.endsWith('.ts');
    if (isPython) {
        return { runner: pythonRunner, command: 'python' };
    }
    else if (isRuby) {
        return { runner: rubyRunner, command: 'ruby' };
    }
    else if (isNode) {
        return { runner: nodeRunner, command: 'node' };
    }
    throw Error(`Unsupported file extension ${stepFilePath}`);
};
const callStepFile = (stepPath, step, event, eventManager, state) => {
    return new Promise((resolve, reject) => {
        const jsonData = JSON.stringify({ ...event });
        const { runner, command } = getLanguageBasedRunner(stepPath);
        const child = (0, child_process_1.spawn)(command, [runner, stepPath, jsonData], {
            stdio: [undefined, undefined, undefined, 'ipc'],
        });
        const rpcProcessor = new step_handler_rpc_processor_1.RpcProcessor(child);
        rpcProcessor.handler('log', async (input) => {
            event.logger.log(input);
        });
        rpcProcessor.handler('state.get', (input) => state.get(input.traceId, input.key));
        rpcProcessor.handler('state.set', (input) => state.set(input.traceId, input.key, input.value));
        rpcProcessor.handler('state.delete', (input) => state.delete(input.traceId, input.key));
        rpcProcessor.handler('state.clear', (input) => state.clear(input.traceId));
        rpcProcessor.handler('emit', (input) => {
            return eventManager.emit({
                ...input,
                traceId: event.traceId,
                flows: event.flows,
                logger: event.logger,
            }, stepPath);
        });
        rpcProcessor.init();
        child.stdout?.on('data', (data) => {
            try {
                const message = JSON.parse(data.toString());
                event.logger.log(message);
            }
            catch (error) {
                event.logger.info(Buffer.from(data).toString(), { step });
            }
        });
        child.stderr?.on('data', (data) => event.logger.error(Buffer.from(data).toString(), { step }));
        child.on('close', (code) => {
            if (code !== 0) {
                reject(new Error(`Process exited with code ${code}`));
            }
            else {
                resolve();
            }
        });
    });
};
exports.callStepFile = callStepFile;
